#include "testApp.h"


bool pointInPolygon(float x, float y, vector<ofPoint>pts);
bool pointInPolygon(float x, float y, vector<ofPoint>pts){
	
	// code via Randolph Franklin...
	int i, j, c = 0;
	int nPts = pts.size();
	
	for (i = 0, j = nPts-1; i < nPts; j = i++) {
		if ((((pts[i].y <= y) && (y < pts[j].y)) ||
			 ((pts[j].y <= y) && (y < pts[i].y))) &&
			(x < (pts[j].x - pts[i].x) * (y - pts[i].y) / (pts[j].y - pts[i].y) + pts[i].x))
			c = !c;
	}
	return (bool) c;
}


//--------------------------------------------------------------
void testApp::setup(){	
	
	
	
	contourSize = 0;

	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	// setup vector field
	VF.setupField(50, 40,ofGetWidth(), ofGetHeight());
	VF.randomizeField(1.5);	
	
	
	#ifdef _USE_LIVE_VIDEO
		vidGrabber.setVerbose(true);
		vidGrabber.initGrabber(320,240);
	#else
		vidPlayer.loadMovie("pinkbear.mov");
		vidPlayer.play();
	#endif
		
    colorImg.allocate(320,240);
	grayImage.allocate(320,240);
	grayBg.allocate(320,240);
	grayDiff.allocate(320,240);
	
	bLearnBakground = true;
	threshold = 30;
	bDrawDiagnostic = false;
	
	ofSetCircleResolution(10);

}

//--------------------------------------------------------------
void testApp::update(){
	
	// fade vector field
	VF.fadeField(0.79f);
	
	
	//---------------------------------------------
	ofBackground(80, 80, 80);
	

	//----------------------------------------------------- video stuff
	bool bNewFrame = false;
	
	#ifdef _USE_LIVE_VIDEO
		vidGrabber.grabFrame();
		bNewFrame = vidGrabber.isFrameNew();
	#else
		vidPlayer.idleMovie();
		bNewFrame = vidPlayer.isFrameNew();
	#endif
	
	if (bNewFrame){
		
		#ifdef _USE_LIVE_VIDEO
				colorImg.setFromPixels(vidGrabber.getPixels(), 320,240);
				colorImg.mirror(false, true);
		#else
				colorImg.setFromPixels(vidPlayer.getPixels(), 320,240);
		#endif
		
        grayImage = colorImg;
		if (bLearnBakground == true){
			grayBg = grayImage;		// the = sign copys the pixels from grayImage into grayBg (operator overloading)
			bLearnBakground = false;
		}
		
		// take the abs value of the difference between background and incoming and then threshold:
		grayDiff.absDiff(grayBg, grayImage);
		grayDiff.threshold(threshold);
		contourFinder.findContours(grayDiff, 25,grayDiff.width * grayDiff.height, 1, false, false);
		
		contourParticles.clear();
		
		if (contourFinder.nBlobs > 0){
			
			for (int i = 0; i < contourFinder.blobs[0].nPts; i+=4){
				
				float scaleX = (float) ofGetWidth() / (float)grayDiff.width;
				float scaleY = (float) ofGetHeight() / (float)grayDiff.height;
				particle p;
				p.setInitialCondition( contourFinder.blobs[0].pts[i].x*scaleX, contourFinder.blobs[0].pts[i].y*scaleY, 0, 0);
				p.radius = 1;  // contourParticles radius
				contourParticles.push_back(p);
			}
		}
	}
	

	
}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofEnableAlphaBlending();
	
	// draw white force field
	VF.draw();
	for (int i = 0; i < contourParticles.size(); i++){
		
		VF.addOutwardCircle(contourParticles[i].pos.x,contourParticles[i].pos.y, 300, 0.3) ;
		
		// draw contour particles
		contourParticles[i].draw();

	}
	

	ofEnableAlphaBlending();
	
	// if there are contour particles
	if (contourFinder.nBlobs > 0){
		glPushMatrix();
		
		glScalef((float) ofGetWidth() / (float)grayDiff.width, (float) ofGetHeight() / (float)grayDiff.height, 1.0f);
		//contourFinder.blobs[0].draw(0,0);
		glPopMatrix();
	}
	ofFill();
	
	if (bDrawDiagnostic == true){
		
		// draw the incoming, the grayscale, the bg and the thresholded difference
		ofSetColor(0xffffff);
		colorImg.draw(20,20);	
		grayImage.draw(360,20);
		grayBg.draw(20,280);
		grayDiff.draw(360,280);
		
    } 		

	/*
	 // display background/video screens
	ofSetColor(0,130,130, 200);
	ofRect(10,550,330,130);
	ofSetColor(0,0,0);
	ofDrawBitmapString("press ' ' (space) to snap background\npress 'd' to toggle diagnostic\ndrag mouse to add particles\npress 'r' to reset particles\npress 't' to toggle force direction", 20, 600);
	 */	
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 

	if (key == ' ' ){
		bLearnBakground = true;
	} 
	
	else if (key == 'd'){	
		bDrawDiagnostic = !bDrawDiagnostic;
	} 
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
